from kubespawner.spawner import KubeSpawner
from traitlets import Unicode


class EODHSpawner(KubeSpawner):
    workspace = Unicode(
        None,
        allow_none=False,
        config=True,
        help="The workspace of the notebook",
    )

    def _expand_user_properties(self, template, slug_scheme=None):
        """
        Expand user properties in template strings
        """
        ns = dict(
            username=self.user.name,
            workspace=self.workspace,
        )
        for attr_name in ("pod_name", "pvc_name", "namespace"):
            ns[attr_name] = getattr(self, attr_name, f"{attr_name}_unavailable!")
        return template.format(**ns)



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

# Configuration file for JupyterHub
import os

c = get_config()  # noqa: F821

# We rely on environment variables to configure JupyterHub so that we
# avoid having to rebuild the JupyterHub container every time we change a
# configuration parameter.

# Spawn single-user servers as Docker containers
c.JupyterHub.spawner_class = "dockerspawner.DockerSpawner"

# Spawn containers from this image
c.DockerSpawner.image = os.environ["DOCKER_NOTEBOOK_IMAGE"]

# Connect containers to this Docker network
network_name = os.environ["DOCKER_NETWORK_NAME"]
c.DockerSpawner.use_internal_ip = True
c.DockerSpawner.network_name = network_name

# Explicitly set notebook directory because we'll be mounting a volume to it.
# Most `jupyter/docker-stacks` *-notebook images run the Notebook server as
# user `jovyan`, and set the notebook directory to `/home/jovyan/work`.
# We follow the same convention.
notebook_dir = os.environ.get("DOCKER_NOTEBOOK_DIR", "/home/jovyan/work")
c.DockerSpawner.notebook_dir = notebook_dir

# Mount the real user's Docker volume on the host to the notebook user's
# notebook directory in the container
c.DockerSpawner.volumes = {"jupyterhub-user-{username}": notebook_dir}

# Remove containers once they are stopped
c.DockerSpawner.remove = True

# For debugging arguments passed to spawned containers
c.DockerSpawner.debug = True

# User containers will access hub by container name on the Docker network
c.JupyterHub.hub_ip = "workspace1.jupyterhub"
c.JupyterHub.hub_port = 8080

# Persist hub data on volume mounted inside container
c.JupyterHub.cookie_secret_file = "/data/jupyterhub_cookie_secret"
c.JupyterHub.db_url = "sqlite:////data/jupyterhub.sqlite"

# Allow all signed-up users to login
c.Authenticator.allow_all = True
c.Authenticator.auto_login = True
c.Authenticator.enable_auth_state = True

# Authenticate users with GenericOAuthenticator
# c.JupyterHub.authenticator_class = "oauthenticator.generic.GenericOAuthenticator"
# c.GenericOAuthenticator.client_id = os.environ["OAUTH_CLIENT_ID"]
# c.GenericOAuthenticator.client_secret = os.environ["OAUTH_CLIENT_SECRET"]
# c.GenericOAuthenticator.authorize_url = os.environ["OAUTH_AUTHORIZE_URL"]
# c.GenericOAuthenticator.token_url = os.environ["OAUTH_TOKEN_URL"]
# c.GenericOAuthenticator.oauth_callback_url = os.environ["OAUTH_CALLBACK_URL"]
# c.GenericOAuthenticator.userdata_url = os.environ["OAUTH_USERDATA_URL"]
# c.GenericOAuthenticator.username_key = os.environ.get(
#     "OAUTH_USERNAME_KEY", "preferred_username"
# )
# c.GenericOAuthenticator.scope = ["openid"]

# Authenticate users with EODHAuthenticator
import re

c.JupyterHub.authenticator_class = "eodh_jpyauth.EODHAuthenticator"
c.EODHAuthenticator.client_id = os.environ["OAUTH_CLIENT_ID"]
c.EODHAuthenticator.client_secret = os.environ["OAUTH_CLIENT_SECRET"]
c.EODHAuthenticator.authorize_url = os.environ["OAUTH_AUTHORIZE_URL"]
c.EODHAuthenticator.token_url = os.environ["OAUTH_TOKEN_URL"]
c.EODHAuthenticator.oauth_callback_url = os.environ["OAUTH_CALLBACK_URL"]
c.EODHAuthenticator.userdata_url = os.environ["OAUTH_USERDATA_URL"]
c.EODHAuthenticator.username_claim = os.environ.get(
    "OAUTH_USERNAME_CLAIM", "preferred_username"
)
c.EODHAuthenticator.scope = ["openid"]
c.EODHAuthenticator.enable_logout = True
c.EODHAuthenticator.oauth_logout_url = os.environ["OAUTH_LOGOUT_URL"]
c.EODHAuthenticator.oauth_logout_redirect_uri = os.environ["OAUTH_LOGOUT_REDIRECT_URI"]
c.EODHAuthenticator.workspace_pattern = re.compile(
    rf"^(?P<workspace>[^.]+)\.{os.environ['PLATFORM_DOMAIN']}$"
)



from oauthenticator.oauth2 import OAuthenticator, OAuthLogoutHandler
from tornado.web import RequestHandler
from traitlets import Bool, CRegExp, Unicode


class EODHLogoutHandler(OAuthLogoutHandler):
    """Log a user out by clearing both their JupyterHub login cookie and SSO cookie."""

    async def get(self):
        self.log.info("EODH Logout")
        if self.authenticator.enable_logout:
            await self.default_handle_logout()
            await self.handle_logout()

            redirect_url = self.authenticator.oauth_logout_url
            if self.authenticator.oauth_logout_redirect_uri:
                redirect_url += (
                    f"?redirect_uri={self.authenticator.oauth_logout_redirect_uri}"
                )

            self.redirect(redirect_url)
        else:
            await super().get()


class EODHAuthenticator(OAuthenticator):
    enable_logout = Bool(False, config=True)
    oauth_logout_url = Unicode(config=True)
    oauth_logout_redirect_uri = Unicode(config=True)
    workspace_pattern = CRegExp(config=True)

    logout_handler = EODHLogoutHandler

    def get_workspace(self, host: str) -> str | None:
        """
        Use authenticator configured compiled regex to extract workspaces from the
        login request host.
        """
        self.log.info("host %s", host)
        matches = self.workspace_pattern.match(host)
        if matches:
            return matches.group("workspace")
        else:
            return None

    def get_callback_url(self, handler: RequestHandler = None) -> str:
        """
        Get my OAuth redirect URL

        Check if the callback_url has a {workspace} placeholder and replace it with the
        workspace extracted from the handler's request host.

        Otherwise pass through to superclass.
        """
        if handler and "{host}" in self.oauth_callback_url:
            return self.oauth_callback_url.format(host=handler.request.host)
        return super().get_callback_url(handler)
